﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Threading;
using System.Runtime.InteropServices;
using LibUsbDotNet;
using LibUsbDotNet.Info;
using LibUsbDotNet.Main;
using System.Collections.ObjectModel;

namespace USBTest_CSharp
{
    class Program
    {
        public static UsbDevice MyUsbDevice;
        public static UsbDeviceFinder MyUsbFinder = new UsbDeviceFinder(0x16C0, 0x05DC);

        public const byte CMD_LED_OFF = 0;
        public const byte CMD_LED_ON = 1;
        public const byte CMD_DATA_OUT = 2;
        public const byte CMD_DATA_WRITE = 3;
        public const byte CMD_DATA_IN = 4;

        // From UsbSetupPacket RequestType docs (or libusb headers or USB spec)
        public const byte USB_DEVICE_TO_HOST = 128;
        public const byte USB_HOST_TO_DEVICE = 0;
        public const byte USB_TYPE_VENDOR = 64;
        public const byte USB_RECIP_DEVICE = 0;

        public static void Main(string[] args)
        {
            ErrorCode ec = ErrorCode.None;

            byte[] bytes = new byte[1024];
            // http://stackoverflow.com/questions/537573/how-to-get-intptr-from-byte-in-c-sharp
            IntPtr buffer = Marshal.AllocHGlobal(bytes.Length);
            int lengthTransferred, actualLength;

            try
            {
                // Find and open the usb device.
                MyUsbDevice = UsbDevice.OpenUsbDevice(MyUsbFinder);

                // If the device is open and ready
                if (MyUsbDevice == null) throw new Exception("Device Not Found.");

                // If this is a "whole" usb device (libusb-win32, linux libusb-1.0)
                // it exposes an IUsbDevice interface. If not (WinUSB) the 
                // 'wholeUsbDevice' variable will be null indicating this is 
                // an interface of a device; it does not require or support 
                // configuration and interface selection.
                IUsbDevice wholeUsbDevice = MyUsbDevice as IUsbDevice;
                if (!ReferenceEquals(wholeUsbDevice, null))
                {
                    // This is a "whole" USB device. Before it can be used, 
                    // the desired configuration and interface must be selected.
                    Console.WriteLine("Whole USB device...");

                    // Select config #1
                    wholeUsbDevice.SetConfiguration(1);

                    // Claim interface #0.
                    wholeUsbDevice.ClaimInterface(0);
                }

                // LED ON
                UsbSetupPacket LED_ON = new UsbSetupPacket(USB_DEVICE_TO_HOST + USB_TYPE_VENDOR + USB_RECIP_DEVICE,
                    CMD_LED_ON, 0, 0, 1024);

                Console.WriteLine("LED ON...");
                MyUsbDevice.ControlTransfer(ref LED_ON, buffer, bytes.Length, out lengthTransferred);

                Thread.Sleep(2500);

                // LED OFF
                UsbSetupPacket LED_OFF = new UsbSetupPacket(USB_DEVICE_TO_HOST + USB_TYPE_VENDOR + USB_RECIP_DEVICE,
                   CMD_LED_OFF, 0, 0, 1024);

                MyUsbDevice.ControlTransfer(ref LED_OFF, buffer, bytes.Length, out lengthTransferred);
                Console.WriteLine("...LED OFF");
                Thread.Sleep(2500);
                // DATA IN
                UsbSetupPacket DATA_IN = new UsbSetupPacket(USB_HOST_TO_DEVICE + USB_TYPE_VENDOR + USB_RECIP_DEVICE,
                     CMD_DATA_IN, 0, 0, 1024);
                string str = "JokkeBK C#";
                byte[] utf8 = System.Text.Encoding.UTF8.GetBytes(str);

                Buffer.BlockCopy(utf8, 0, bytes, 0, utf8.Length); // copy utf-8 encoded string to bytes
                bytes[utf8.Length] = 0; // NULL terminate to maintain compatibility with C client        
                Marshal.Copy(bytes, 0, buffer, utf8.Length + 1); // copy null-terminated string to IntPtr

                MyUsbDevice.ControlTransfer(ref DATA_IN, buffer, utf8.Length + 1, out lengthTransferred);

                Console.WriteLine("Wrote " + lengthTransferred + " bytes (" + utf8.Length +
                    " chars and NULL) with DATA IN: " + str);
                Thread.Sleep(5000);
                // DATA OUT
                UsbSetupPacket DATA_OUT = new UsbSetupPacket(USB_DEVICE_TO_HOST + USB_TYPE_VENDOR + USB_RECIP_DEVICE,
                     CMD_DATA_OUT, 0, 0, 1024);

                MyUsbDevice.ControlTransfer(ref DATA_OUT, buffer, bytes.Length, out lengthTransferred);
                Marshal.Copy(buffer, bytes, 0, lengthTransferred); // copy transferred bytes from IntPtr to bytes
                // Find the terminating NULL and thus actual length
                for (actualLength = 0; actualLength < lengthTransferred && bytes[actualLength] != 0; actualLength++)
                {
                }

                Console.WriteLine("Got " + lengthTransferred + " bytes (" + actualLength +
                    " chars and NULL) with DATA OUT: " + System.Text.Encoding.UTF8.GetString(bytes, 0, actualLength));

                // Wait for user input..
                Console.WriteLine("Taste drucken");
                Console.ReadKey();
            }
            catch (Exception ex)
            {
                Console.WriteLine();
                Console.WriteLine((ec != ErrorCode.None ? ec + ":" : String.Empty) + ex.Message);
            }
            finally
            {
                if (MyUsbDevice != null)
                {
                    if (MyUsbDevice.IsOpen)
                    {
                        // If this is a "whole" usb device (libusb-win32, linux libusb-1.0)
                        // it exposes an IUsbDevice interface. If not (WinUSB) the 
                        // 'wholeUsbDevice' variable will be null indicating this is 
                        // an interface of a device; it does not require or support 
                        // configuration and interface selection.
                        IUsbDevice wholeUsbDevice = MyUsbDevice as IUsbDevice;
                        if (!ReferenceEquals(wholeUsbDevice, null))
                        {
                            // Release interface #0.
                            wholeUsbDevice.ReleaseInterface(0);
                        }

                        MyUsbDevice.Close();
                    }
                    MyUsbDevice = null;

                    // Free usb resources
                    UsbDevice.Exit();

                    Marshal.FreeHGlobal(buffer);
                }
            }
        }
    }
}
